package monads;

import java.util.Optional;

import static monads.util.OptionalUtils.optionalAdd;

public class OptionalExamples {

    private final Soundcard soundcard;
    private final Computer computer;
    private final USB usb;

    public OptionalExamples() {
        usb = new USB("3.1");
        soundcard = new Soundcard(usb);
        computer = new Computer(soundcard);
    }

    private void nestedOptional() {
        System.out.println("Nested optionals example");
        var version = computer.getSoundcard()
                .flatMap(Soundcard::getUSB)
                .map(USB::getVersion)
                .orElse("UNKNOWN");
        System.out.printf("Version : %s%n", version);
    }

    private void addOptionals() {
        System.out.println("Add optionals example");
        assert optionalAdd(Optional.of(1), Optional.of(1)).equals(Optional.of(2));
        assert optionalAdd(Optional.of(1), Optional.of(1)).equals(Optional.empty());
    }

    public static void main(String[] args) {
        var examples = new OptionalExamples();
        examples.nestedOptional();
        examples.addOptionals();
    }

    private static class Computer {
        private final Soundcard soundcard;

        private Computer(Soundcard soundcard) {
            this.soundcard = soundcard;
        }

        public Optional<Soundcard> getSoundcard() { return Optional.ofNullable(soundcard); }
    }

    private static class Soundcard {
        private final USB usb;

        private Soundcard(USB usb) {
            this.usb = usb;
        }

        public Optional<USB> getUSB() { return Optional.ofNullable(usb); }
    }

    private static class USB {
        private final String version;

        private USB(String version) {
            this.version = version;
        }

        public String getVersion(){ return version; }
    }
}
