package monads;

import java.util.stream.Stream;

public class StreamExamples {
    /**
     * Given n > 0 find all pairs (i; j) where
     * 1 < j <= i <= n and i + j is prime
     */
    private Stream<long[]> findPrimes(long n) {
        return Stream.iterate(1, i -> i + 1)
                .limit(n)
                .flatMap(i -> Stream.iterate(1, j -> j + 1)
                        .limit(n)
                        .map(j -> new long[] {i, j})
                ).filter(pair -> isPrime(pair[0] + pair[1]));
    }

    private boolean isPrime(long n) {
        return Stream.iterate(2, i -> i + 1)
                .limit((long) Math.sqrt(n))
                .noneMatch(i -> n % i == 0);
    }

    public static void main(String[] args) {
        StreamExamples examples = new StreamExamples();
        var start = System.nanoTime();
        var primes = (Long) examples.findPrimes(10L).count();
        var end = System.nanoTime();
        System.out.printf("Elapsed : %fs", (end - start) / 1_000_000_000.0);
        //primes.map(Arrays::toString).forEach(System.out::println);
    }
}
